<?php

namespace Oxapay\Woocommerce;

use WC_Payment_Gateway;
final class oxapayGateway extends WC_Payment_Gateway
{
    /**
     * @var string
     */
    public $id = 'oxapay';
    /**
     * @var bool
     */
    public $has_fields = true;
    /**
     * @var string
     */
    public $plugin_name = 'OxaPay Payment Method for WooCommerce';
    /**
     * @var string
     */
    public $title = 'Pay with OxaPay';
    /**
     * @var string
     */
    public $method_title = 'OxaPay';
    /**
     * @var string
     */
    public $method_description = "";
    /**
     * @var string
     */
    public $merchant;
    /**
     * @var string
     */
    public $lifetime;
        /**
     * @var string
     */
    public $currency;



    public function __construct()
    {
        $this->title = $this->get_option('title') ?: $this->title;
        $this->method_description = "
        <img src='https://radiant-flame-44830ef920.media.strapiapp.com/64f99bd74a_1711559826645_a36469cf13.jpeg' style='max-width: 500px; height: auto;'>
        <p>To start using OxaPay Crypto Payment Gateway, you need to follow a fewsimple steps.</p>
        <p>1. Create an account on <a href='https://app.oxapay.com/signup'>OxaPay.com</a>.</p>
        <p>2. Navigate to the Merchants API tab in your account.</p>
        <p>3. Click on the 'Generate Merchant API Key' button.</p>
        <p>4. Give your Merchant API key a meaningful name for identification purposes.</p>
        ";
        $this->plugin_name = $this->get_option('plugin_name');
        $this->method_title = $this->get_option('method_title');
        $this->form_fields = $this->adminFields();
        $this->has_fields = false;
        $this->init_settings();
        $this->title = $this->get_option('title');
        $this->description = $this->get_option('description');
        $this->merchant = $this->get_option('merchant');
        $this->lifetime = $this->get_option('lifetime');
        $this->currency = $this->get_option('currency');
        add_action('woocommerce_api_oxapay', array($this, 'check_oxapay_response'));
        add_action('woocommerce_update_options_payment_gateways_oxapay', array($this, 'process_admin_options'));


    }
    /**
     * @return array
     */

    public function adminFields()
    {
        if (empty($this->get_option('merchant'))) {
            $merchant = 'sandbox';
        }
        else  {
            $merchant = $this->get_option('merchant');

        }      
        $url = "https://api.oxapay.com/merchants/allowedCoins";
        
        $data = array(
            "merchant" => $merchant
        );
        
        $options = array(
            'http' => array(
                'header' => 'Content-Type: application/json',
                'method' => 'POST',
                'content' => json_encode($data),
            ),
        );
        
        $context  = stream_context_create($options);
        $response = file_get_contents($url, false, $context);
        $currencies = array('' => 'Select By User');
        $coins = json_decode($response);
        if ($coins->result == 100)
        {
            foreach ($coins->allowed as $coin) {
                $currencies[$coin] = ($coin);
            }
        }

        return [
            'enabled' => [
                'title' => __('Enabled'),
                'type' => 'checkbox',
                'default' => 'no',
                'desc_tip' => true
            ],
            'merchant' => [
                'title' => 'OxaPay Merchant Key
                <p><font size="1">You can find your OxaPay Merchant Key in the settings of your personal account. Read more <a href="https://docs.oxapay.com/integrations/merchant-api">here</a>.</font></p>
                ',
                'type' => 'text',
                'default' => 'sadnbox'
            ],
            'title' => [
                'title' => 'Method title
                <p><font size="1">Payment method name. For example "OxaPay".</font></p>',
                'type' => 'text',
                'default' => 'Pay with Oxapay'
            ],
            'description' => [
                'title' => 'Payment Instruction',
                'type' => 'text',
                'default' => 'Crypto Payment Gateway'
            ],
            'currency' => [
                'title' => 'Currency (Optional)
                <p><font size="1">If you want your invoice to be paid in a specified currency (TRX, ETH, ...) and set the price base on it choose it otherwise fill it blank.</font></p>
                ',
                'type' => 'select',
                'options' => $currencies,
            ],
            'lifetime' => [
                'title' => 'LifeTime
                <p>Set the expiration time for the payment link in minutes (15-2880).</p>
                ',
                'type' => 'number',
                'default' => '60',
                'custom_attributes' => [
                    'min' => 15,
                    'max' => 2880,
                ],
            ],
        ];
    }
    public function admin_options()
    {
        echo '<h3>'.'OxaPay Payment Gateway'.'</h3>';
        echo '<table class="form-table">';
        $this->generate_settings_html();
        echo '</table>';
    }

    /**
     * @param $order_id
     * @return array
     */
    public function process_payment($order_id) {
        $order = wc_get_order($order_id);
        $order->update_status(paymentStatus::WC_STATUS_PENDING);
        $order->save();
        wc_reduce_stock_levels($order_id);
        WC()->cart->empty_cart();
        $returnUrl = $this->get_return_url($order);
        $data = array(
            'merchant'    => $this->merchant,
            'lifeTime'    => (int)$this->lifetime,
            'amount'      => $order->get_total(),
            'orderId'     => strval($order_id),
            'callbackUrl' => get_site_url(null, "wc-api/oxapay"),
            'returnUrl'   => $returnUrl,
        );
        if ($this->currency) {
            $b['currency'] = $this->currency;
        }
        $result = wp_remote_post('https://api.oxapay.com/merchants/request', [
            'timeout' => 25,
            'body' => wp_json_encode($data),
            'headers' => [
                'Content-Type' => 'application/json',
                'Accept' => 'application/json',
            ],
            'sslverify' => false,
        ]);

        if (is_wp_error($result))
        {
            $order->add_order_note($message);
            $order->update_status(paymentStatus::WC_STATUS_FAIL);
            $order->save();
        }
        $pay = json_decode(wp_remote_retrieve_body($result));
        if ($pay->result == 100)
        {
            update_post_meta($order_id, 'oxapay_trackid', $pay->trackId);
            session_write_close();
            echo '<p>'.'Thank you for your order. you are redirecting to OxaPay gateway. please wait'.'</p>';
            return ['result' => 'success', 'redirect' => $pay->payLink];
        }
        else
        {
            $order->add_order_note($message);
            $order->update_status(paymentStatus::WC_STATUS_FAIL);
            $order->save();
        }
        return ['result' => 'success', 'redirect' => $this->get_return_url($order)];
        }
    function check_oxapay_response()
		{

			$postData = file_get_contents('php://input');
			$data = json_decode($postData);
            if ($data->type === 'payment') {
                $apiSecretKey = $this->merchant;
            } 
            else {
                http_response_code(400);
                echo 'Invalid data.type';
                exit;
            }
            $hmacHeader = $_SERVER['HTTP_HMAC'];
            $calculatedHmac = hash_hmac('sha512', $postData, $apiSecretKey);
            if ($calculatedHmac === $hmacHeader) {
                if (empty($data->orderId)) {
                    http_response_code(400);
                    echo 'orderId is required';
                    exit;
                }
                if (empty($data->status)) {
                    http_response_code(400);
                    echo 'status is required';
                    exit;
                }

                $order = wc_get_order($data->orderId);
                $order->set_status(paymentStatus::convertToWoocommerceStatus($data->status));
                $order->save();
                
                echo 'Received payment callback: ' . json_encode($data);
                http_response_code(200);
                echo 'OK';
                exit;

            }
            else {
                http_response_code(400);
                echo 'Invalid HMAC signature';
            }
                                

        
		}

}